$(document).ready(function(){
	SERGIOPEREIRA.samples.todaysBrew.init();
});

var SERGIOPEREIRA = {
	samples: {
		todaysBrew: {
			state: { },
			feedUrl: 'http://feeds.feedburner.com/ReflectivePerspective?format=xml',
			todaysUrl: '',
			maxAgeHours: 12, // keep it for 12 hours
			
			init: function(){
				$('#content h3').live('click', function(){
					$('#content ul:visible').slideUp();
					$(this).next().slideToggle();
				});

				$('#content li a').live('click', function(){
					$('#content ul:visible').slideUp();
					chrome.tabs.create({url: this.href});
				});

				$('h1').click(function(){
					chrome.tabs.create({url: SERGIOPEREIRA.samples.todaysBrew.todaysUrl});
				});
				
				if(typeof chrome != 'undefined') {
					this.state = localStorage.getItem('latestBrew');

					if(this.state){
						var now = new Date();
						var minTimestamp = new Date();
						minTimestamp.setHours(minTimestamp.getHours() - this.maxAgeHours);
						minTimestamp = minTimestamp.toJSON();

						if(this.state.timestamp > minTimestamp) {
							this.renderLatest(this.state.latestData);
							return;
						}
					}

					this.refresh();

				}
			},
			refresh: function(){
				console.log('will get feed data...');
				$.get(this.feedUrl, function(xml, status, xhr){
					SERGIOPEREIRA.samples.todaysBrew.update(xml);
				});
			},
			update: function(feedXml) {
				var latest = this.getFirstItem(feedXml);
				this.state = { };
				this.state.latestData = latest;
				this.state.timestamp = new Date();
				localStorage['latestBrew'] = JSON.stringify(this.state);
				this.renderLatest(latest);
			},
			renderLatest: function(latest){
				$('#main>h1').text(latest.title);
				$('#content').html(latest.content);
				this.todaysUrl = latest.url;
			},			
			getFirstItem: function(feedXml){
				var items = feedXml.evaluate("//channel/item", feedXml, null, XPathResult.ANY_TYPE, null); 

				var item = items.iterateNext(); 
				if (item) {
					return this.createItem(item);
				}
			},			
			createItem: function(postXml) {
				return { 
					title: this.readElementText(postXml, 'title'),
					url: this.readElementText(postXml, 'feedburner:origLink'),
					content: this.readElementText(postXml, 'content:encoded')
				};
			},
			mapElements: function(contextElement, path, map){
				var result = [ ];
				var items = contextElement.ownerDocument.evaluate(path, contextElement, this.namespaceResolver, XPathResult.ANY_TYPE, null); 
				
				var item = items.iterateNext(); 
				var i = 0;
				while (item) {
					result.push( map(item, i++) );
					item = items.iterateNext();
				}

				return result;
			},			
			readElementText: function(contextElement, path){
				var results = contextElement.ownerDocument.evaluate(path, contextElement, this.namespaceResolver, XPathResult.ANY_TYPE, null); 
				var first = results.iterateNext(); 

				if (first) {
					return first.textContent;
				}
			},			
			namespaceResolver: function(prefix) {
				if(prefix == 'content') {
					return 'http://purl.org/rss/1.0/modules/content/';
				}
				if(prefix == 'feedburner') {
					return 'http://rssnamespace.org/feedburner/ext/1.0';
				}
			}

		}
	}
};
